@@ -1,14 +1,17 @@
 package Graph::Weighted;
+BEGIN {
+  $Graph::Weighted::AUTHORITY = 'cpan:GENE';
+}
 # ABSTRACT: A weighted graph implementation
 
+our $VERSION = '0.53';
+
 use warnings;
 use strict;
 
-our $VERSION = '0.5201';
-
 use base qw(Graph);
 
-use constant DEBUG => 0;
+use constant DEBUG  => 0;
 use constant WEIGHT => 'weight';
 
 sub new {
@@ -19,13 +22,11 @@ sub new {
 }
 
 sub populate {
-    my ($self, $data, $attr, $method) = @_;
+    my ($self, $data, $attr, $vertex_method, $edge_method) = @_;
     warn "populate(): $data\n" if DEBUG;
 
     # Set the default attribute.
     $attr ||= WEIGHT;
-    # Set the default method.
-    $method ||= undef;
 
     # What type of data are we given?
     my $data_ref = ref $data;
@@ -34,14 +35,18 @@ sub populate {
         my $vertex = 0; # Initial vertex id.
         for my $neighbors (@$data) {
             warn "Neighbors of $vertex: [@$neighbors]\n" if DEBUG;
-            $self->_add_weighted_edges_from_array($vertex, $neighbors, $attr, $method);
+            $self->_add_weighted_edges_from_array(
+                $vertex, $neighbors, $attr, $vertex_method, $edge_method
+            );
             $vertex++; # Move on to the next vertex...
         }
     }
     elsif ($data_ref eq 'HASH') {
         for my $vertex (keys %$data) {
             warn "Neighbors of $vertex: [", join(' ', values %{$data->{$vertex}}), "]\n" if DEBUG;
-            $self->_add_weighted_edges_from_hash($vertex, $data->{$vertex}, $attr, $method);
+            $self->_add_weighted_edges_from_hash(
+                $vertex, $data->{$vertex}, $attr, $vertex_method, $edge_method
+            );
         }
     }
     else {
@@ -50,23 +55,29 @@ sub populate {
 }
 
 sub _add_weighted_edges_from_array {
-    my ($self, $vertex, $neighbors, $attr, $method) = @_;
+    my ($self, $vertex, $neighbors, $attr, $vertex_method, $edge_method) = @_;
     warn "add_weighted_edges(): $vertex, $neighbors, $attr\n" if DEBUG;
+
     # Initial vertex weight
     my $vertex_weight = 0;
+
     # Make nodes and edges.
     for my $n (0 .. @$neighbors - 1) {
         my $w = $neighbors->[$n]; # Weight of the edge to the neighbor.
         next unless $w; # TODO Skip zero weight nodes if requested?
+
         # Add a node-node edge to the graph.
         $self->add_edge($vertex, $n);
+
         # Set the weight of the edge.
-        my $edge_weight = _compute_edge_weight($w, $attr, $method);
+        my $edge_weight = _compute_edge_weight($w, $attr, $edge_method);
         warn "Edge: $vertex -($edge_weight)-> $n\n" if DEBUG;
         $self->set_edge_attribute($vertex, $n, $attr, $edge_weight);
+
         # Tally the weight of the vertex.
-        $vertex_weight = _compute_vertex_weight($w, $vertex_weight, $attr, $method);
+        $vertex_weight = _compute_vertex_weight($w, $vertex_weight, $attr, $vertex_method);
     }
+
     # Set the weight of the graph node.
     warn "Vertex $vertex $attr = $vertex_weight\n" if DEBUG;
     $self->set_vertex_attribute($vertex, $attr, $vertex_weight);
@@ -75,20 +86,32 @@ sub _add_weighted_edges_from_array {
 sub _add_weighted_edges_from_hash {
     my ($self, $vertex, $neighbors, $attr, $method) = @_;
     warn "add_weighted_edges(): $vertex, $neighbors, $attr\n" if DEBUG;
+
     # Initial vertex weight
     my $vertex_weight = 0;
-    # Make nodes and edges.
-    for my $n (keys %$neighbors) {
-        my $w = $neighbors->{$n}; # Weight of the edge to the neighbor.
-        # Add a node-node edge to the graph.
-        $self->add_edge($vertex, $n);
-        # Set the weight of the edge.
-        my $edge_weight = _compute_edge_weight($w, $attr, $method);
-        warn "Edge: $vertex -($edge_weight)-> $n\n" if DEBUG;
-        $self->set_edge_attribute($vertex, $n, $attr, $edge_weight);
-        # Tally the weight of the vertex.
-        $vertex_weight = _compute_vertex_weight($w, $vertex_weight, $attr, $method);
+
+    # Handle terminal nodes.
+    if (ref $neighbors) {
+        # Make nodes and edges.
+        for my $n (keys %$neighbors) {
+            my $w = $neighbors->{$n}; # Weight of the edge to the neighbor.
+
+            # Add a node-node edge to the graph.
+            $self->add_edge($vertex, $n);
+
+            # Set the weight of the edge.
+            my $edge_weight = _compute_edge_weight($w, $attr, $method);
+            warn "Edge: $vertex -($edge_weight)-> $n\n" if DEBUG;
+            $self->set_edge_attribute($vertex, $n, $attr, $edge_weight);
+
+            # Tally the weight of the vertex.
+            $vertex_weight = _compute_vertex_weight($w, $vertex_weight, $attr, $method);
+        }
     }
+    else {
+        $vertex_weight = $neighbors;
+    }
+
     # Set the weight of the graph node.
     warn "Vertex $vertex $attr = $vertex_weight\n" if DEBUG;
     $self->set_vertex_attribute($vertex, $attr, $vertex_weight);
@@ -97,8 +120,10 @@ sub _add_weighted_edges_from_hash {
 sub _compute_edge_weight {
     my ($weight, $attr, $method) = @_;
     warn "compute_edge_weight(): $attr $weight\n" if DEBUG;
+
     # Call the weight function if one is given.
     return $method->($weight, $attr) if $method and ref $method eq 'CODE';
+
     # Increment the current value by the node weight if no weight function is given.
     return $weight;
 }
@@ -106,8 +131,10 @@ sub _compute_edge_weight {
 sub _compute_vertex_weight {
     my ($weight, $current, $attr, $method) = @_;
     warn "compute_vertex_weight(): $attr $weight, $current\n" if DEBUG;
+
     # Call the weight function if one is given.
     return $method->($weight, $current, $attr) if $method and ref $method eq 'CODE';
+
     # Increment the current value by the node weight if no weight function is given.
     return $weight + $current;
 }
@@ -116,26 +143,35 @@ sub get_weight {
     my $self = shift;
     return $self->get_attr(@_);
 }
+
 sub get_attr {
     my ($self, $v, $attr) = @_;
     die 'ERROR: No vertex given to get_attr()' unless defined $v;
+
+    # Default to weight.
     $attr ||= WEIGHT;
     warn "get_attr($v, $attr)\n" if DEBUG;
-    if (ref $v eq 'ARRAY') {
-        return $self->get_edge_attribute(@$v, $attr) || 0;
-    }
-    else {
-        return $self->get_vertex_attribute($v, $attr) || 0;
-    }
+
+    # Return the edge attribute if given a list.
+    return $self->get_edge_attribute(@$v, $attr) || 0 if ref $v eq 'ARRAY';
+
+    # Return the vertex attribute if given a scalar.
+    return $self->get_vertex_attribute($v, $attr) || 0;
 }
 
 1;
-__END__
+
+
+=pod
 
 =head1 NAME
 
 Graph::Weighted - A weighted graph implementation
 
+=head1 VERSION
+
+version 0.53
+
 =head1 SYNOPSIS
 
   use Graph::Weighted;
@@ -153,8 +189,9 @@ Graph::Weighted - A weighted graph implementation
     { 0 => { 1=>2, 2=>1 },             # Vertex with 2 edges of weight 3
       1 => { 0=>3, 2=>1 },             #      "      2        "        4
       2 => { 0=>3, 1=>2 },             #      "      2        "        5
-      3 => { 1=>2 },                   #      "      1        "        2
+      3 => { 1=>2, 5=>1 },             #      "      2        "        3
       4 => { 0=>1, 1=>1, 2=>1, 3=>1 }, #      "      4        "        4
+      5 => 3.1415,                     # A terminal node!
     },
     $attr
   );
@@ -179,15 +216,19 @@ attributes.  As such, all of the C<Graph> methods may be used as
 documented.  This module is a streamlined version of the weight based
 accessors provided by the C<Graph> module.
 
+=head1 NAME
+
+Graph::Weighted - A weighted graph implementation
+
 =head1 METHODS
 
-=head2 new(%arguments)
+=head2 new()
 
 Return a new C<Graph::Weighted> object.
 
 See L<Graph> for the possible constructor arguments.
 
-=head2 populate($data, $attribute, $method)
+=head2 populate()
 
   $g->populate(\@vectors)
   $g->populate(
@@ -199,65 +240,73 @@ See L<Graph> for the possible constructor arguments.
     },
   );
 
-Arguments:
+Populate a graph with weighted nodes.  Arguments:
 
-  data      => Array ref of numeric vectors or HASH ref of numeric edge values
-  attribute => Optional string name
-  method    => Optional code ref weighting function
+  data          => Array ref of numeric vectors or HASH ref of numeric edge values
+  attribute     => Optional string name
+  vertex_method => Optional code ref weighting function
+  edge_method   => Optional code ref weighting function
 
-Populate a graph with weighted nodes.
+Examples of C<data> in array reference form:
 
-Example of array reference nodes:
-
-  []      no edges
+  []      No edges
   [0]     1 vertex and 1 edge to node 0 (weight 0)
-  [1]     1 vertex and 1 edge to node 0 (vertex & edge weights 1)
+  [1]     1 vertex and 1 edge to node 0 (vertex & edge weight 1)
   [0,1]   2 vertices and 2 edges (edge weights 0,1; vertex weight 1)
   [0,1,9] 3 vertices and 3 edges (edge weights 0,1,9; vertex weight 10)
 
-The default edge weighting function returns the value in the neighbor
-position.  An alternative may be provided, as a subroutine reference,
-which should accept the current edge weight and the attribute to update.
-For example:
+Data can also be given explicitly as a hash reference of nodes and
+edges.
 
-  sub edge_weight_function {
-    my ($weight, attribute);
-    return $current_weight_total / $current_node_weight;
-  }
+The C<attribute> is named 'weight' by default, but may be anything of
+your choosing.  This method can be called multiple times on the same
+graph, for nodes of the same name but different attributes values.
 
-The default vertex weighting function is a simple sum of the neighbor
-weight values.  An alternative may be provided, which should accept
-of the current node weight, current weight total and the
-attribute as arguments to update.  For example:
+The default vertex weighting function (given by C<vertex_method>) is a
+simple sum of the neighbor weights.  An alternative may be provided,
+which should accept of the current node weight, current weight total
+and the attribute as arguments to update.  For example:
 
   sub vertex_weight_function {
     my ($current_node_weight, $current_weight_total, attribute);
     return $current_weight_total / $current_node_weight;
   }
 
-The attribute is named 'weight' by default but may be anything of
-your choosing.  This method can be called multiple times on the same
-graph, for nodes of different attributes.
+The default edge weighting function (given by C<edge_method>) simply
+returns the value in the node's neighbor position.  An alternative may
+be provided, as a subroutine reference, which should accept the
+current edge weight and the attribute to update.  For example:
 
-=head2 get_weight($vertex), get_weight($edge), get_attr($vertex, $attribute), get_attr($edge, $attribute)
+  sub edge_weight_function {
+    my ($weight, attribute);
+    return $current_weight_total / $current_node_weight;
+  }
 
-  $g->get_weight($vertex);
-  $g->get_attr($vertex, $attribute);
-  $g->get_weight(\@edge);
-  $g->get_attr(\@edge, $attribute);
+=head2 get_weight()
 
-Return the attribute value for the vertex or edge.  A vertex is a
-numeric value.  An edge is an array reference with 2 elements.
+  $w = $g->get_weight($vertex);
+  $w = $g->get_weight(\@edge);
+
+Return the weight for the vertex or edge.
+
+A vertex is a numeric value.  An edge is an array reference with 2
+elements.  If no value is found, zero is returned.
+
+=head2 get_attr()
+
+  $w = $g->get_attr($vertex, $attribute);
+  $w = $g->get_attr(\@edge, $attribute);
+
+Return the named attribute value for the vertex or edge or zero.
 
 =head1 TO DO
 
-Accept hashrefs and C<Matrix::*> objects instead of just LoLs.
+Accept hashrefs and C<Matrix::*> objects instead of just LoLs.  
+Also, possibly L<Statistics::Descriptive::Weighted>.
 
-Also, possibly L<Statistics::Descriptive::Weighted>
+Find the heaviest and lightest nodes.
 
-Make subroutines for finding the heaviest and lightest nodes.
-
-Make subroutines for finding the total weight beneath a node.
+Find the total weight beneath a node.
 
 =head1 SEE ALSO
 
@@ -267,14 +316,29 @@ The F<eg/> and F<t/*> sources.
 
 =head1 AUTHOR
 
-Gene Boggs, C<< <gene at cpan.org> >>
+Gene Boggs, E<lt>gene@cpan.orgE<gt>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2003-2012 Gene Boggs.
+Copyright 2003-2012 Gene Boggs
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
 by the Free Software Foundation; or the Artistic License.
 
+=head1 AUTHOR
+
+Gene Boggs <gene@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2012 by Gene Boggs.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
+
+
+__END__
+
